/*
 * GuildsEngine
 *
 * @author: MarWit :: AMXX.pl < http://amxx.pl/user/13472-marwit/ >
*/

#include < amxmodx >
#include < nvault >
#include < nvault_util >
#include < ColorChat >
#include < hamsandwich >

#include "diablo_nowe.inc"

enum pluginCvarsStruct {
	pcCanCreateGuild,
	pcCreateGuildLevel,
	pcGoldPerKill
}

enum guildInfoStruct {
	giName[ 64 ],
	giGold,
	giHealth,
	giExperienceBoost,
	giSpeedBoost,
	Array: giMembers
}

enum ( <<= 1 ) {
	grLeadder = ( 1 << 1 ),
	grViceLeadder,
	grMember
}

enum playerInfoStruct {
	miRank,
	miGuild[ 64 ],
	miGold
}

enum { VAULT_GUILDS = 0, VAULT_PLAYERS }

new gVault[ 2 ];
new giLastGuildIndex = 0;

new gPluginCvars[ pluginCvarsStruct ];
new gPlayerInfo[ 33 ][ playerInfoStruct ];

new Trie: gtGuilds, Array: gaGuilds;
new Trie: gtPlayers;

new playerGuildBuffer[ 33 ][ guildInfoStruct ];

public plugin_init( )
{
	static const VERSION[ ] = "0.1" ;
	
	register_plugin( "Guilds Engine", VERSION, "MarWit :: AMXX.pl" );
	register_cvar( "guilds_version", VERSION, FCVAR_SPONLY | FCVAR_SERVER );
	
	gPluginCvars[ pcCanCreateGuild ] = register_cvar( "guilds_cancreateguild", "1" );
	gPluginCvars[ pcCreateGuildLevel ] = register_cvar( "guilds_createguildlevel", "60" );
	gPluginCvars[ pcGoldPerKill ] = register_cvar( "guilds_goldperkill", "1" );
	
	RegisterHam( Ham_Spawn, "player", "hamPlayerSpawned", 1 );
	RegisterHam( Ham_Killed, "player", "hamPlayerKilledPost", 1 );

	register_clcmd( "say /guild", "cmdGuild" );
	
	register_clcmd( "nazwaGildii", "cmdGuildCreate" );
	register_clcmd( "ileZlota", "cmdDeposit" );
	register_clcmd( "amx_dajzloto", "cmdGiveGold", ADMIN_IMMUNITY );
	
	logToFile( "Plugin Init" );
}

public plugin_precache( )
	loadGuilds();

public plugin_end( )
{
	nvault_close( gVault[ VAULT_PLAYERS ] );
	saveGuilds( );
	
	logToFile( "Plugin End" );
}

public plugin_natives( )
{
	register_native( "diablo_set_user_gold", "nativeSetGold" );
	register_native( "diablo_get_user_gold", "nativeGetGold" );
}

public diablo_hud_write( id, szData[ ], iLen )
{
	if( TrieKeyExists( gtGuilds, gPlayerInfo[ id ][ miGuild ] ) )
	{
		TrieGetArray( gtGuilds, gPlayerInfo[ id ][ miGuild ], playerGuildBuffer[ id ], guildInfoStruct );
		format( szData, iLen, "%s^n[Gildia: %s]^n[Zloto: %d]", szData, playerGuildBuffer[ id ][ giName ], gPlayerInfo[ id ][ miGold ] );
	}
	else
		format( szData, iLen, "%s^n[Gildia: Brak]^n[Zloto: %d]", szData, gPlayerInfo[ id ][ miGold ] );
}

public client_connect( id ) 
{
	if( is_user_bot( id ) ) return;
	
	client_cmd( id, "bind ^"v^" ^"say /guild^"" );
	
	gPlayerInfo[ id ][ miRank ] = -1;
	copy( gPlayerInfo[ id ][ miGuild ], 63, "-1" );
	gPlayerInfo[ id ][ miGold ] = 0;

	new szNick[ 33 ]
	get_user_name( id, szNick, 32 );
	
	if( TrieKeyExists( gtPlayers, szNick ) )
		TrieGetArray( gtPlayers, szNick, gPlayerInfo[ id ], playerInfoStruct );
		
	logToFile( "Client connect: %s", szNick );
}

public client_disconnect( id )
{
	if( is_user_bot( id ) ) return;
	
	new szNick[ 33 ]
	get_user_name( id, szNick, 32 );
	
	TrieSetArray( gtPlayers, szNick, gPlayerInfo[ id ], playerInfoStruct );
	savePlayer( id );
	
	logToFile( "Client disconnect: %s", szNick );
}

public hamPlayerSpawned( id )
{
	if( ! TrieKeyExists( gtGuilds, gPlayerInfo[ id ][ miGuild ] ) )
		return;

	new guildBuffer[ guildInfoStruct ];
	TrieGetArray( gtGuilds, gPlayerInfo[ id ][ miGuild ], guildBuffer, guildInfoStruct );

	diablo_add_speed( id, guildBuffer[ giSpeedBoost ] * 8.5  );
	diablo_add_hp( id, guildBuffer[ giHealth ] * 5 );
}

public hamPlayerKilledPost( iThis, iKiller, iShouldgibs )
{
	if( ! is_user_connected( iKiller ) || iThis == iKiller )
		return;
		
	gPlayerInfo[ iKiller ][ miGold ] += get_pcvar_num( gPluginCvars[ pcGoldPerKill ] );
		
	if( ! TrieKeyExists( gtGuilds, gPlayerInfo[ iKiller ][ miGuild ] ) )
		return;
		
	new guildBuffer[ guildInfoStruct ];
	TrieGetArray( gtGuilds, gPlayerInfo[ iKiller ][ miGuild ], guildBuffer, guildInfoStruct );
	diablo_add_xp( iKiller, guildBuffer[ giExperienceBoost ] * 300 );
}

public savePlayer( id )
{
	if( is_user_bot( id ) ) return;
	
	new szNick[ 33 ], szBuffer[ 512 ];
	get_user_name( id, szNick, 32 );
	
	TrieSetArray( gtPlayers, szNick, gPlayerInfo[ id ], playerInfoStruct );
	
	formatex( szBuffer, 511, "%d %s %d", 
							gPlayerInfo[ id ][ miRank ],
							gPlayerInfo[ id ][ miGuild ],
							gPlayerInfo[ id ][ miGold ]
	);
	
	nvault_set( gVault[ VAULT_PLAYERS ], szNick, szBuffer );
	
	logToFile( "Save Player: %s | %s", szNick, szBuffer );
	
	gPlayerInfo[ id ][ miRank ] = -1;
	copy( gPlayerInfo[ id ][ miGuild ], 63, "-1" );
	gPlayerInfo[ id ][ miGold ] = 0;
}

public loadGuilds( )
{
	gVault[ VAULT_GUILDS ] = nvault_util_open( "Guilds" ); 
	gVault[ VAULT_PLAYERS ] = nvault_util_open( "GuildsPlayers" ); 
	
	gtGuilds = TrieCreate( );
	gaGuilds = ArrayCreate( 64 );
	gtPlayers = TrieCreate( );
	
	new iPos, szKey[ 64 ], szBuffer[ 512 ], iTimestamp;
	new iCount = nvault_util_count( gVault[ VAULT_GUILDS ] );
	
	new playerBuffer[ playerInfoStruct ];
	new guildBuffer[ guildInfoStruct ];
	
	new i;
	new szArgs[ 5 ][ 64 ]; 
	
	for( i = 1; i <= iCount; i ++ )
	{
		iPos = nvault_util_read( gVault[ VAULT_GUILDS ], iPos, szKey, 63, szBuffer, 511, iTimestamp );
		
		logToFile( "Load Guild: %s | %s", szKey, szBuffer );
		
		parse( szBuffer, 
				szArgs[ 0 ], 63,
				szArgs[ 1 ], 63,
				szArgs[ 2 ], 63,
				szArgs[ 3 ], 63,
				szArgs[ 4 ], 63
		);
		
		copy( guildBuffer[ giName ], 63, szKey );
		
		guildBuffer[ giGold ] = str_to_num( szArgs[ 1 ] );
		guildBuffer[ giHealth ] = str_to_num( szArgs[ 2 ] );
		guildBuffer[ giExperienceBoost ] = str_to_num( szArgs[ 3 ] );
		guildBuffer[ giSpeedBoost ] = str_to_num( szArgs[ 4 ] );
		guildBuffer[ giMembers ] = _: ArrayCreate( 33 );

		if( str_to_num( szArgs[ 0 ] ) > giLastGuildIndex )
			giLastGuildIndex = str_to_num( szArgs[ 0 ] );
		
		TrieSetArray( gtGuilds, szArgs[ 0 ], guildBuffer, guildInfoStruct );
		
		ArrayPushString( gaGuilds, szArgs[ 0 ] );
	}
	
	iPos = 0;
	iCount = nvault_util_count( gVault[ VAULT_PLAYERS ] );
	
	for( i = 1; i <= iCount; i ++ )
	{
		iPos = nvault_util_read( gVault[ VAULT_PLAYERS ], iPos, szKey, 63, szBuffer, 511, iTimestamp );
		
		logToFile( "Load Player: %s | %s", szKey, szBuffer );
		
		if( ! szKey[ 0 ] )
			continue;
		
		parse( szBuffer, 
			szArgs[ 0 ], 63,
			playerBuffer[ miGuild ], 63,
			szArgs[ 1 ], 63
		);
		
		playerBuffer[ miRank ] = str_to_num( szArgs[ 0 ] );
		playerBuffer[ miGold ] = str_to_num( szArgs[ 1 ] );
		
		if( TrieKeyExists( gtGuilds, playerBuffer[ miGuild ] ) || playerBuffer[ miGold ] )
		{
			if( TrieKeyExists( gtGuilds, playerBuffer[ miGuild ] ) )
			{
				TrieGetArray( gtGuilds, playerBuffer[ miGuild ], guildBuffer, guildInfoStruct );
				ArrayPushString( guildBuffer[ giMembers ], szKey );
			}
			
			TrieSetArray( gtPlayers, szKey, playerBuffer, playerInfoStruct );
		}
	}
	
	nvault_util_close( gVault[ VAULT_GUILDS ] );
	nvault_util_close( gVault[ VAULT_PLAYERS ] );
	
	gVault[ VAULT_PLAYERS ] = nvault_open( "GuildsPlayers" ); 
	nvault_prune( gVault[ VAULT_PLAYERS ], 0, get_systime( ) - ( 86400 * 30 ) );
}

public saveGuilds( )
{
	new szKey[ 64 ], szBuffer[ 512 ];
	gVault[ VAULT_GUILDS ] = nvault_open( "Guilds" );

	for( new i = 0 ; i < ArraySize( gaGuilds ); i ++ )
	{
		ArrayGetString( gaGuilds, i, szKey, 63 );
		
		new guildBuffer[ guildInfoStruct ];
		TrieGetArray( gtGuilds, szKey, guildBuffer, guildInfoStruct );
		TrieDeleteKey( gtGuilds, szKey );
		
		ArrayDestroy( guildBuffer[ giMembers ] );
		
		formatex( szBuffer, 511, "%s %d %d %d %d", 
					szKey,
					guildBuffer[ giGold ],
					guildBuffer[ giHealth ],
					guildBuffer[ giExperienceBoost ],
					guildBuffer[ giSpeedBoost ]
		);
		
		logToFile( "Save Guild: %s | %s", guildBuffer[ giName ], szBuffer );
		
		nvault_set( gVault[ VAULT_GUILDS ], guildBuffer[ giName ], szBuffer );
	}

	nvault_close( gVault[ VAULT_GUILDS ] );
	
	TrieDestroy( gtGuilds )
	ArrayDestroy( gaGuilds );
	TrieDestroy( gtPlayers );
}

public cmdGuild( id )
{
	new iMenu = menu_create( "Guilds Menu:", "cmdGuildH" );
	
	if( ! TrieKeyExists( gtGuilds, gPlayerInfo[ id ][ miGuild ] ) && ! get_pcvar_num( gPluginCvars[ pcCanCreateGuild ] ) )
		return;
	
	if( ! TrieKeyExists( gtGuilds, gPlayerInfo[ id ][ miGuild ] ) )
		menu_additem( iMenu, "\rStworz Guild'ie\g ( 200 zlota )", "0" );
	else
	{
		if( gPlayerInfo[ id ][ miRank ] & ( grLeadder | grViceLeadder ) )
		{
			menu_additem( iMenu, "\rDodaj czlonka do Gildii", "1" );
			menu_additem( iMenu, "\rUsun czlonka z Gildii", "2" );
			
			if( gPlayerInfo[ id ][ miRank ] & grLeadder )
			{
				menu_additem( iMenu, "\rUstaw range dla czlonka", "3" );
				menu_additem( iMenu, "\rRozwiaz Gildie", "4" );
				menu_additem( iMenu, "\rSkille Gildii", "5" );
			}
		}
		
		if( gPlayerInfo[ id ][ miRank ] & grMember )
			menu_additem( iMenu, "\rOpusc Gildie", "6" );
			
		menu_additem( iMenu, "\yCzlonkowie Gildii", "7" );
		menu_additem( iMenu, "\yWplac zloto", "8" );
	}
	
	menu_setprop( iMenu, MPROP_EXITNAME, "\yWyjdz^n^n\wCopyright by MarWit 2013" );
	menu_display( id, iMenu );
}

public cmdGuildH( id, iMenu, iItem )
{
	new iAccess, iCallback;
	new szItem[ 5 ];
	
	menu_item_getinfo( iMenu, iItem, iAccess, szItem, 4, .callback=iCallback );
	new guildBuffer[ guildInfoStruct ], szNick[ 33 ], szBuffer[ 512 ];
	
	if( iItem == MENU_EXIT )
		return;
	
	switch( str_to_num( szItem ) )
	{
		case 0: {
			if( diablo_get_user_level(id) < get_pcvar_num( gPluginCvars[ pcCreateGuildLevel ] ) )
				ColorChat( id, TEAM_COLOR, "[^x04 Guild^x03 ]^x01 Twoj poziom jest zbyt niski." );
			else if( gPlayerInfo[ id ][ miGold ] >= 200 )
				client_cmd( id, "messagemode nazwaGildii" );
			else
				ColorChat( id, TEAM_COLOR, "[^x04 Guild^x03 ]^x01 Nie posiadasz wystarczajaco zlota." );
		}
		case 1: {
			new iMenu = menu_create( "Guilds Menu - Dodaj czlonka:", "addMemberH" );
			
			for( new i = 1; i < 32; i ++ )
			{
				if( !is_user_connected( i ) || TrieKeyExists( gtGuilds, gPlayerInfo[ i ][ miGuild ] ) )
					continue;
					
				new szID[ 5 ];
				num_to_str( i, szID, 4 );
				get_user_name( i, szNick, 32 );
				
				menu_additem( iMenu, szNick, szID );
			}
			
			menu_setprop( iMenu, MPROP_EXITNAME, "\yWyjdz" );
			
			menu_display( id, iMenu );
		}
		case 2: {
			new iMenu = menu_create( "Guilds Menu - Usun czlonka:", "removeMemberH" );
			
			TrieGetArray( gtGuilds, gPlayerInfo[ id ][ miGuild ], guildBuffer, guildInfoStruct );
			
			for( new i = 0 ; i < ArraySize( guildBuffer[ giMembers ] ); i ++ )
			{
				new szID[ 5 ];
				num_to_str( i, szID, 4 );
			
				ArrayGetString( guildBuffer[ giMembers ], i, szNick, 32 );
				menu_additem( iMenu, szNick, szID );
			}
			
			menu_setprop( iMenu, MPROP_EXITNAME, "\yWyjdz" );
			
			menu_display( id, iMenu );
		}
		case 3: {
			new iMenu = menu_create( "Guilds Menu - Ustaw Range:", "setMemberRankH" );
			
			TrieGetArray( gtGuilds, gPlayerInfo[ id ][ miGuild ], guildBuffer, guildInfoStruct );
			
			for( new i = 0 ; i < ArraySize( guildBuffer[ giMembers ] ); i ++ )
			{
				ArrayGetString( guildBuffer[ giMembers ], i, szNick, 32 );
				menu_additem( iMenu, szNick );
			}
			
			menu_setprop( iMenu, MPROP_EXITNAME, "\yWyjdz" );
			
			menu_display( id, iMenu );
		}
		case 4: {
			new iMenu = menu_create( "Guilds Menu - Czy na pewno chcesz rozwiazac gildie?:", "removeGuildH" );
			
			menu_additem( iMenu, "Tak" );
			menu_setprop( iMenu, MPROP_EXITNAME, "\rNie" );
			
			menu_display( id, iMenu );
		}
		case 5: guildSkillsMenu( id );
		case 6: {
			get_user_name( id, szNick, 32 );
			TrieGetArray( gtGuilds, gPlayerInfo[ id ][ miGuild ], guildBuffer, guildInfoStruct );
			
			for( new i = 0; i < ArraySize( guildBuffer[ giMembers ] ); i++ )
			{
				ArrayGetString( guildBuffer[ giMembers ], i, szBuffer, 511 );
				if( equali( szBuffer, szNick ) )
				{
					ArrayDeleteItem( guildBuffer[ giMembers ], i );
					break;
				}
			}
			
			TrieSetArray( gtGuilds, gPlayerInfo[ id ][ miGuild ], guildBuffer, guildInfoStruct );
			format( gPlayerInfo[ id ][ miGuild ], 63, "-1" );
			gPlayerInfo[ id ][ miRank ] = -1;
			
			ColorChat( id, TEAM_COLOR, "[^x04 Guild^x03 ]^x01 Opusciles gildie." );
		}
		
		case 7: {
			if( !TrieKeyExists( gtGuilds, gPlayerInfo[ id ][ miGuild ] ) )
				return;
			
			ColorChat( id, TEAM_COLOR, "[^x04 Guild^x03 ]^x01 Czlonkowie gildii:" );
			
			TrieGetArray( gtGuilds, gPlayerInfo[ id ][ miGuild ], guildBuffer, guildInfoStruct );
			
			for( new i = 0; i < ArraySize( Array: guildBuffer[ giMembers ] ); i++ )
			{
				ArrayGetString( Array: guildBuffer[ giMembers ], i, szNick, 32 );
				
				if( !i )
					format( szBuffer, 511, "%s", szNick );
				else if( strlen( szBuffer ) + strlen( szNick ) > 192 )
				{
					client_print( id, print_chat, szBuffer );
					format( szBuffer, 511, "%s", szNick );
				}
				else
				{
					add( szBuffer, 511, ", " );
					add( szBuffer, 511, szNick );
				}
				
				if( i + 1  == ArraySize( Array: guildBuffer[ giMembers ] ) )
					client_print( id, print_chat, szBuffer );
			}
		}
		
		case 8: {
			if( TrieKeyExists( gtGuilds, gPlayerInfo[ id ][ miGuild ] ) )
				client_cmd( id, "messagemode ileZlota" );
		}
	}
}

public addMemberH( id, iMenu, iItem )
{
	if( iItem == MENU_EXIT )
		return;

	new szID[ 5 ], szNick[ 33 ], iTarget;
	menu_item_getinfo( iMenu, iItem, iTarget, szID, 4, szNick, 32, iTarget );
	
	iTarget = str_to_num( szID );
	if( ! is_user_connected( iTarget ) )
		return;
		
	sendGuildInvite( id, iTarget );
}

public removeMemberH( id, iMenu, iItem )
{
	if( iItem == MENU_EXIT )
		return;

	new szID[ 5 ], szNick[ 33 ], iTarget;
	menu_item_getinfo( iMenu, iItem, iTarget, szID, 4, szNick, 32, iTarget );
	
	new guildBuffer[ guildInfoStruct ];
	TrieGetArray( gtGuilds, gPlayerInfo[ id ][ miGuild ], guildBuffer, guildInfoStruct );
	
	ArrayDeleteItem( guildBuffer[ giMembers ], str_to_num( szID ) );
	
	iTarget = cmd_target( id, szNick, 0 );
	
	if( is_user_connected( iTarget ) )
	{
		copy( gPlayerInfo[ iTarget ][ miGuild ], 63, "-1" );
		ColorChat( iTarget, TEAM_COLOR, "[^x04 Guild^x03 ]^x01 Zostales wykopany z gidlii !" );
	}
	
	new playerBuffer[ playerInfoStruct ];
	TrieGetArray( gtPlayers, szNick, playerBuffer, playerInfoStruct );
		
	copy( playerBuffer[ miGuild ], 63, "-1" );
	TrieSetArray( gtPlayers, szNick, playerBuffer, playerInfoStruct );
		
	new szBuffer[ 512 ];
	formatex( szBuffer, 511, "%d %s %d", 
						playerBuffer[ miRank ],
						playerBuffer[ miGuild ],
						playerBuffer[ miGold ]
	);
	
	nvault_set( gVault[ VAULT_PLAYERS ], szNick, szBuffer );
}

public setMemberRankH( id, iMenu, iItem )
{
	if( iItem == MENU_EXIT )
		return;

	new szID[ 5 ], szNick[ 33 ], iTarget;
	menu_item_getinfo( iMenu, iItem, iTarget, szID, 4, szNick, 32, iTarget );
	
	iTarget = cmd_target( id, szNick, 0 );
	
	if( ! is_user_connected( iTarget ) )
	{
		ColorChat( iTarget, TEAM_COLOR, "[^x04 Guild^x03 ]^x01 Czlonek musi byc podlaczony do serwera !" );
		return
	}
	
	gPlayerInfo[ iTarget ][ miRank ] = ( ( gPlayerInfo[ iTarget ][ miRank ] & grMember ) ? grLeadder : ( gPlayerInfo[ iTarget ][ miRank ] * 2 ) );
	
	ColorChat( id, TEAM_COLOR, "[^x04 Guild^x03 ]^x01 Ranga %s zostala zmieniona !", szNick );
}

public removeGuildH( id, iMenu, iItem )
{
	if( iItem != 0 )
		return;
		
	new guildBuffer[ guildInfoStruct ];
	
	TrieGetArray( gtGuilds, gPlayerInfo[ id ][ miGuild ], guildBuffer, guildInfoStruct );
	TrieDeleteKey( gtGuilds, gPlayerInfo[ id ][ miGuild ] );
	ArrayDestroy( guildBuffer[ giMembers ] );
	
	gVault[ VAULT_GUILDS ] = nvault_open( "Guilds" );
	nvault_remove( gVault[ VAULT_GUILDS ], gPlayerInfo[ id ][ miGuild ] );
	nvault_close( gVault[ VAULT_GUILDS ] );
	
	copy( gPlayerInfo[ id ][ miGuild ], 63, "-1" );
	ColorChat( id, TEAM_COLOR, "[^x04 Guild^x03 ]^x01 Gildia zostala usunieta !" );
}

public guildSkillsMenu( id )
{
	new iMenu = menu_create( "Guilds Menu - Ustaw skille ( koszt 50 zlota ) :", "guildSkillsMenuH" );
	new szBuffer[ 256 ];
	new guildBuffer[ guildInfoStruct ];
	TrieGetArray( gtGuilds, gPlayerInfo[ id ][ miGuild ], guildBuffer, guildInfoStruct );
	
	formatex( szBuffer, 255, "Exp Booster ( zwieksza ilosc zdobywanego exp'a ) [ %d ]", guildBuffer[ giExperienceBoost ] );
	menu_additem( iMenu, szBuffer );
	
	formatex( szBuffer, 255, "Zycie ( zwieksza ilosc zycia kazdego czlonka ) [ %d ]", guildBuffer[ giHealth ] );
	menu_additem( iMenu, szBuffer );
	
	formatex( szBuffer, 255, "Speed ( zwieksza predkosc kazdego czlonka ) [ %d ]", guildBuffer[ giSpeedBoost ] );
	menu_additem( iMenu, szBuffer );
	
	formatex( szBuffer, 255, "Wyjdz^n^nIlosc zlota: %d", guildBuffer[ giGold ] );
	menu_setprop( iMenu, MPROP_EXITNAME, szBuffer );
	menu_display( id, iMenu );
}

public guildSkillsMenuH( id, iMenu, iItem )
{
	if( iMenu == MENU_EXIT )
	{
		menu_destroy( iMenu );
		return;
	}
	
	new guildBuffer[ guildInfoStruct ];
	TrieGetArray( gtGuilds, gPlayerInfo[ id ][ miGuild ], guildBuffer, guildInfoStruct );
	
	if( guildBuffer[ giGold ] < 50 )
	{
		ColorChat( id, TEAM_COLOR, "[^x04 Guild^x03 ]^x01 Gildia nie posiada tyle zlota !" );
		return;
	}
	
	guildBuffer[ giGold ] -= 50;
	
	switch( iItem )
	{
		case 0: guildBuffer[ giExperienceBoost ] ++;
		case 1: guildBuffer[ giHealth ] ++;
		case 2: guildBuffer[ giSpeedBoost ] ++;
	}
	
	TrieSetArray( gtGuilds, gPlayerInfo[ id ][ miGuild ], guildBuffer, guildInfoStruct );
	
	guildSkillsMenu( id );
}

public sendGuildInvite( iSender, iReciver )
{
	new szBuffer[ 256 ];
	new guildBuffer[ guildInfoStruct ];
	TrieGetArray( gtGuilds, gPlayerInfo[ iSender ][ miGuild ], guildBuffer, guildInfoStruct );
	
	formatex( szBuffer, 255, "Zostales zaproszony do guildi %s..", guildBuffer[ giName ] );
	new iMenu = menu_create( szBuffer, "sendGuildInviteH" );
	
	num_to_str( iSender, szBuffer, 255 );
	
	menu_additem( iMenu, "\yPrzyjmuje zaproszenie", szBuffer );
	menu_additem( iMenu, "\rOdrzucam zaproszenie", szBuffer );
	
	menu_setprop( iMenu, MPROP_EXIT, MEXIT_NEVER );
	menu_display( iReciver, iMenu );
}

public sendGuildInviteH( id, iMenu, iItem )
{
	if( iMenu == MENU_EXIT )
	{
		menu_destroy( iMenu );
		return;
	}

	new szID[ 5 ], szInfo[ 33 ], iSender;
	menu_item_getinfo( iMenu, iItem, iSender, szID, 4, szInfo, 32, iSender );
	
	iSender = str_to_num( szID );
	new szNick[ 33 ];
	get_user_name( id, szNick, 32 );

	switch( iItem )
	{
		case 0: {
			new guildBuffer[ guildInfoStruct ];
			TrieGetArray( gtGuilds, gPlayerInfo[ iSender ][ miGuild ], guildBuffer, guildInfoStruct );
			ArrayPushString( guildBuffer[ giMembers ], szNick );
			
			copy( gPlayerInfo[ id ][ miGuild ], 63, gPlayerInfo[ iSender ][ miGuild ] );
			gPlayerInfo[ id ][ miRank ] = grMember;
			
			ColorChat( iSender, TEAM_COLOR, "[^x04 Guild^x03 ]^x01 %s przyja twoje zaproszenie", szNick );
		}
		
		case 1: ColorChat( iSender, TEAM_COLOR, "[^x04 Guild^x03 ]^x01 %s odrzucil twoje zaproszenie.", szNick );
	}
}

public cmdGuildCreate( id )
{
	if( TrieKeyExists( gtGuilds, gPlayerInfo[ id ][ miGuild ] ) || gPlayerInfo[ id ][ miGold ] < 200 )
		return;

	new szBuffer[ 64 ];
	read_argv( 1, szBuffer, 63 );
	remove_quotes( szBuffer );
	
	if( isGuildExists( szBuffer ) )
	{
		ColorChat( id, TEAM_COLOR, "[^x04 Guild^x03 ]^x01 Gildia o takiej nazwie ( %s ) juz istnieje!", szBuffer );
		return;
	}
	
	gPlayerInfo[ id ][ miGold ] -= 200;
	gPlayerInfo[ id ][ miRank ] = grLeadder;
	giLastGuildIndex ++;
	
	new guildBuffer[ guildInfoStruct ];
	copy( guildBuffer[ giName ], 63, szBuffer );
	
	guildBuffer[ giMembers ] = _: ArrayCreate( 33 );
	
	get_user_name( id, szBuffer, 32 );
	ArrayPushString( guildBuffer[ giMembers ], szBuffer );
	
	num_to_str( giLastGuildIndex, gPlayerInfo[ id ][ miGuild ], 63 );
	TrieSetArray( gtGuilds, gPlayerInfo[ id ][ miGuild ], guildBuffer, guildInfoStruct );
	
	ArrayPushString( gaGuilds, gPlayerInfo[ id ][ miGuild ] );
	
	ColorChat( id, TEAM_COLOR, "[^x04 Guild^x03 ]^x01 Gildia o nazwie %s zostala zalozona!", guildBuffer[ giName ] );
}

public cmdDeposit( id )
{
	if( !TrieKeyExists( gtGuilds, gPlayerInfo[ id ][ miGuild ] ) || !gPlayerInfo[ id ][ miGold ] )
		return;
		
	new szValue[ 8 ];
	read_argv( 1, szValue, 7 );
	remove_quotes( szValue );
	
	if( str_to_num( szValue ) > gPlayerInfo[ id ][ miGold ] )
	{
		ColorChat( id, TEAM_COLOR, "[^x04 Guild^x03 ]^x01 Nie posiadasz tyle zlota! Sproboj jeszcze raz." );
		client_cmd( id, "messagemode ileZlota" );
		
		return;
	}
	
	gPlayerInfo[ id ][ miGold ] -= str_to_num( szValue );
	
	new guildBuffer[ guildInfoStruct ];
	TrieGetArray( gtGuilds, gPlayerInfo[ id ][ miGuild ], guildBuffer, guildInfoStruct );
	
	guildBuffer[ giGold ] += str_to_num( szValue );
	TrieSetArray( gtGuilds, gPlayerInfo[ id ][ miGuild ], guildBuffer, guildInfoStruct );
	
	ColorChat( id, TEAM_COLOR, "[^x04 Guild^x03 ]^x01 Zloto pomyslnie wplacone! Aktualny stan skarbca: %d", guildBuffer[ giGold ] );
}

public cmdGiveGold( id, iLevel, iCid )
{
	if( !cmd_access( id, iLevel, iCid, 2 ) )
		return;
		
	new szTarget[ 33 ], szNum[ 5 ];
	
	read_argv( 1, szTarget, 32 );
	read_argv( 2, szNum, 4 );
	
	remove_quotes( szTarget );
	remove_quotes( szNum );
	
	new iTarget = cmd_target( id, szTarget, 0 );
	if( ! iTarget )
		return;
		
	gPlayerInfo[ iTarget ][ miGold ] += str_to_num( szNum );
}

public nativeSetGold( iPlugin, iParams )
{
	if( iParams != 2 || !is_user_connected( get_param( 1 ) ) )
		return -1;
		
	gPlayerInfo[ get_param( 1 ) ][ miGold ] = get_param( 2 );
	return gPlayerInfo[ get_param( 1 ) ][ miGold ];
}

public nativeGetGold( iPlugin, iParams )
{
	if( iParams != 1 || !is_user_connected( get_param( 1 ) ) )
		return -1;
		
	return gPlayerInfo[ get_param( 1 ) ][ miGold ];
}

stock logToFile( szMessage[ ], ... )
{
	new szMsg[ 512 ];
	vformat( szMsg, 511, szMessage, 2 );
	
	write_file( "guilds_log.txt", szMsg );
}

stock isGuildExists( szName[ ] )
{
	new szBuffer[ 64 ];
	new guildBuffer[ guildInfoStruct ];
	
	for( new i = 0 ; i < ArraySize( gaGuilds ); i ++ )
	{
		ArrayGetString( gaGuilds, i, szBuffer, 63 );
		TrieGetArray( gtGuilds, szBuffer, guildBuffer, guildInfoStruct );
	
		if( equali( szName, guildBuffer[ giName ] ) )
			return true;
	}
	
	return false;
}